@@ -182,7 +182,7 @@ struct supplicant_task {
 	char *netpath;
 	gboolean created;
 	enum supplicant_state state;
-	gboolean noscan;
+	gboolean scanning;
 	GSList *scan_results;
 	struct iw_range *range;
 	gboolean disconnecting;
@@ -250,27 +250,26 @@ static int get_range(struct supplicant_task *task)
 	return err;
 }
 
-static char *get_bssid(struct connman_device *device)
+static int get_bssid(struct connman_device *device,
+				unsigned char *bssid, unsigned int *bssid_len)
 {
-	char *bssid;
-	unsigned char ioctl_bssid[ETH_ALEN];
-	int ifindex;
-	char *ifname;
 	struct iwreq wrq;
+	char *ifname;
+	int ifindex;
 	int fd, err;
 
 	ifindex = connman_device_get_index(device);
 	if (ifindex < 0)
-		return NULL;
+		return -EINVAL;
 
 	ifname = connman_inet_ifname(ifindex);
 	if (ifname == NULL)
-		return NULL;
+		return -EINVAL;
 
 	fd = socket(PF_INET, SOCK_DGRAM, 0);
 	if (fd < 0) {
 		g_free(ifname);
-		return NULL;
+		return -EINVAL;
 	}
 
 	memset(&wrq, 0, sizeof(wrq));
@@ -282,20 +281,12 @@ static char *get_bssid(struct connman_device *device)
 	close(fd);
 
 	if (err < 0)
-		return NULL;
+		return -EIO;
 
-	memcpy(ioctl_bssid, wrq.u.ap_addr.sa_data, ETH_ALEN);
+	memcpy(bssid, wrq.u.ap_addr.sa_data, ETH_ALEN);
+	*bssid_len = ETH_ALEN;
 
-	bssid = g_try_malloc0(13);
-	if (bssid == NULL)
-		return NULL;
-
-	snprintf(bssid, 13, "%02x%02x%02x%02x%02x%02x",
-					ioctl_bssid[0], ioctl_bssid[1],
-					ioctl_bssid[2], ioctl_bssid[3],
-					ioctl_bssid[4], ioctl_bssid[5]);
-
-	return bssid;
+	return 0;
 }
 
 static void add_interface_reply(DBusPendingCall *call, void *user_data)
@@ -338,6 +329,8 @@ static void add_interface_reply(DBusPendingCall *call, void *user_data)
 	return;
 
 failed:
+	dbus_message_unref(reply);
+
 	task_list = g_slist_remove(task_list, task);
 
 	connman_device_unref(task->device);
@@ -359,6 +352,8 @@ static int add_interface(struct supplicant_task *task)
 	if (message == NULL)
 		return -ENOMEM;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	dbus_message_iter_init_append(message, &array);
 
 	dbus_message_iter_append_basic(&array,
@@ -447,6 +442,8 @@ static int create_interface(struct supplicant_task *task)
 	if (message == NULL)
 		return -ENOMEM;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	dbus_message_append_args(message, DBUS_TYPE_STRING, &task->ifname,
 							DBUS_TYPE_INVALID);
 
@@ -497,16 +494,20 @@ static int remove_interface(struct supplicant_task *task)
 
 	DBG("task %p", task);
 
+#if 0
 	if (task->created == FALSE) {
 		connman_device_set_powered(task->device, FALSE);
 		return 0;
 	}
+#endif
 
 	message = dbus_message_new_method_call(SUPPLICANT_NAME, SUPPLICANT_PATH,
 					SUPPLICANT_INTF, "removeInterface");
 	if (message == NULL)
 		return -ENOMEM;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	dbus_message_append_args(message, DBUS_TYPE_OBJECT_PATH, &task->path,
 							DBUS_TYPE_INVALID);
 
@@ -544,6 +545,8 @@ static int set_ap_scan(struct supplicant_task *task)
 	if (message == NULL)
 		return -ENOMEM;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	dbus_message_append_args(message, DBUS_TYPE_UINT32, &ap_scan,
 							DBUS_TYPE_INVALID);
 
@@ -585,6 +588,8 @@ static int add_network(struct supplicant_task *task)
 	if (message == NULL)
 		return -ENOMEM;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	dbus_error_init(&error);
 
 	reply = dbus_connection_send_with_reply_and_block(connection,
@@ -638,6 +643,8 @@ static int remove_network(struct supplicant_task *task)
 	if (message == NULL)
 		return -ENOMEM;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	dbus_message_append_args(message, DBUS_TYPE_OBJECT_PATH, &task->netpath,
 							DBUS_TYPE_INVALID);
 
@@ -680,6 +687,8 @@ static int select_network(struct supplicant_task *task)
 	if (message == NULL)
 		return -ENOMEM;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	dbus_message_append_args(message, DBUS_TYPE_OBJECT_PATH, &task->netpath,
 							DBUS_TYPE_INVALID);
 
@@ -719,6 +728,8 @@ static int enable_network(struct supplicant_task *task)
 	if (message == NULL)
 		return -ENOMEM;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	dbus_error_init(&error);
 
 	reply = dbus_connection_send_with_reply_and_block(connection,
@@ -755,6 +766,8 @@ static int disable_network(struct supplicant_task *task)
 	if (message == NULL)
 		return -ENOMEM;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	dbus_error_init(&error);
 
 	reply = dbus_connection_send_with_reply_and_block(connection,
@@ -796,6 +809,8 @@ static int set_network(struct supplicant_task *task,
 	if (message == NULL)
 		return -ENOMEM;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	dbus_message_iter_init_append(message, &array);
 
 	dbus_message_iter_open_container(&array, DBUS_TYPE_ARRAY,
@@ -888,6 +903,30 @@ static int set_network(struct supplicant_task *task,
 	return 0;
 }
 
+static void scan_reply(DBusPendingCall *call, void *user_data)
+{
+	struct supplicant_task *task = user_data;
+	DBusMessage *reply;
+
+	DBG("task %p", task);
+
+	reply = dbus_pending_call_steal_reply(call);
+	if (reply == NULL)
+		return;
+
+	if (dbus_message_get_type(reply) == DBUS_MESSAGE_TYPE_ERROR) {
+		connman_device_set_scanning(task->device, FALSE);
+		goto done;
+	}
+
+	if (task->scanning == TRUE)
+		connman_device_set_scanning(task->device, TRUE);
+
+done:
+	dbus_message_unref(reply);
+}
+
+
 static int initiate_scan(struct supplicant_task *task)
 {
 	DBusMessage *message;
@@ -900,6 +939,8 @@ static int initiate_scan(struct supplicant_task *task)
 	if (message == NULL)
 		return -ENOMEM;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	if (dbus_connection_send_with_reply(connection, message,
 						&call, TIMEOUT) == FALSE) {
 		connman_error("Failed to initiate scan");
@@ -907,9 +948,17 @@ static int initiate_scan(struct supplicant_task *task)
 		return -EIO;
 	}
 
+	if (call == NULL) {
+		connman_error("D-Bus connection not available");
+		dbus_message_unref(message);
+		return -EIO;
+	}
+
+	dbus_pending_call_set_notify(call, scan_reply, task, NULL);
+
 	dbus_message_unref(message);
 
-	return 0;
+	return -EINPROGRESS;
 }
 
 static struct {
@@ -1336,6 +1385,8 @@ static void get_properties(struct supplicant_task *task)
 	if (message == NULL)
 		goto noscan;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	if (dbus_connection_send_with_reply(connection, message,
 						&call, TIMEOUT) == FALSE) {
 		connman_error("Failed to get network properties");
@@ -1356,7 +1407,10 @@ static void get_properties(struct supplicant_task *task)
 	return;
 
 noscan:
-	connman_device_set_scanning(task->device, FALSE);
+	if (task->scanning == TRUE) {
+		connman_device_set_scanning(task->device, FALSE);
+		task->scanning = FALSE;
+	}
 }
 
 static void scan_results_reply(DBusPendingCall *call, void *user_data)
@@ -1413,7 +1467,10 @@ done:
 	dbus_message_unref(reply);
 
 noscan:
-	connman_device_set_scanning(task->device, FALSE);
+	if (task->scanning == TRUE) {
+		connman_device_set_scanning(task->device, FALSE);
+		task->scanning = FALSE;
+	}
 }
 
 static void scan_results_available(struct supplicant_task *task)
@@ -1429,20 +1486,22 @@ static void scan_results_available(struct supplicant_task *task)
 	if (message == NULL)
 		return;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	if (dbus_connection_send_with_reply(connection, message,
 						&call, TIMEOUT) == FALSE) {
 		connman_error("Failed to request scan result");
 		goto done;
 	}
 
-	if (task->noscan == FALSE)
-		connman_device_set_scanning(task->device, TRUE);
-
 	if (call == NULL) {
 		connman_error("D-Bus connection not available");
 		goto done;
 	}
 
+	if (task->scanning == TRUE)
+		connman_device_set_scanning(task->device, TRUE);
+
 	dbus_pending_call_set_notify(call, scan_results_reply, task, NULL);
 
 done:
@@ -1476,6 +1535,7 @@ static int task_connect(struct supplicant_task *task)
 	const char *address, *security, *passphrase;
 	const void *ssid;
 	unsigned int ssid_len;
+	int err;
 
 	address = connman_network_get_string(task->network, "Address");
 	security = connman_network_get_string(task->network, "WiFi.Security");
@@ -1499,7 +1559,9 @@ static int task_connect(struct supplicant_task *task)
 
 	set_network(task, ssid, ssid_len, address, security, passphrase);
 
-	enable_network(task);
+	err = enable_network(task);
+	if (err < 0)
+		return err;
 
 	return -EINPROGRESS;
 }
@@ -1508,6 +1570,8 @@ static void state_change(struct supplicant_task *task, DBusMessage *msg)
 {
 	DBusError error;
 	const char *newstate, *oldstate;
+	unsigned char bssid[ETH_ALEN];
+	unsigned int bssid_len;
 	enum supplicant_state state;
 
 	dbus_error_init(&error);
@@ -1529,73 +1593,33 @@ static void state_change(struct supplicant_task *task, DBusMessage *msg)
 	if (state == WPA_INVALID)
 		return;
 
-	task->state = state;
-
-	switch (task->state) {
-	case WPA_SCANNING:
-		task->noscan = TRUE;
-		connman_device_set_scanning(task->device, TRUE);
-		break;
-	case WPA_ASSOCIATING:
-	case WPA_ASSOCIATED:
-	case WPA_4WAY_HANDSHAKE:
-	case WPA_GROUP_HANDSHAKE:
-		task->noscan = TRUE;
-		break;
-	case WPA_COMPLETED:
-	case WPA_DISCONNECTED:
-		task->noscan = FALSE;
-		break;
-	case WPA_INACTIVE:
-		task->noscan = FALSE;
+	if (task->scanning == TRUE && state != WPA_SCANNING) {
 		connman_device_set_scanning(task->device, FALSE);
-		break;
-	case WPA_INVALID:
-		break;
+		task->scanning = FALSE;
 	}
 
+	task->state = state;
+
 	if (task->network == NULL)
 		return;
 
 	switch (task->state) {
 	case WPA_COMPLETED:
-		if (connman_network_get_group(task->network) == NULL) {
-			const char *name, *mode, *security;
-			char *bssid;
-
-			/*
-			 * This is a hidden network, we need to set its
-			 * group based on the BSSID we just joined.
-			 */
-			bssid = get_bssid(task->device);
-
-			name = connman_network_get_string(task->network,
-								"Name");
-			mode = connman_network_get_string(task->network,
-								"WiFi.Mode");
-			security = connman_network_get_string(task->network,
-							"WiFi.Security");
-
-			if (bssid && name && mode && security) {
-				char *group;
-
-				group = build_group(bssid, name, NULL, 0,
-								mode, security);
-				connman_network_set_group(task->network, group);
-				g_free(group);
-			}
-
-			g_free(bssid);
-		}
+		if (get_bssid(task->device, bssid, &bssid_len) == 0)
+	                connman_network_set_address(task->network,
+							bssid, bssid_len);
 
 		/* carrier on */
 		connman_network_set_connected(task->network, TRUE);
-		connman_device_set_scanning(task->device, FALSE);
 		break;
 
 	case WPA_DISCONNECTED:
+		disable_network(task);
+
+		/* carrier off */
+		connman_network_set_connected(task->network, FALSE);
+
 		if (task->disconnecting == TRUE) {
-			connman_network_set_connected(task->network, FALSE);
 			connman_network_unref(task->network);
 			task->disconnecting = FALSE;
 
@@ -1604,10 +1628,6 @@ static void state_change(struct supplicant_task *task, DBusMessage *msg)
 				task->pending_network = NULL;
 				task_connect(task);
 			}
-		} else {
-			/* carrier off */
-			connman_network_set_connected(task->network, FALSE);
-			connman_device_set_scanning(task->device, FALSE);
 		}
 		break;
 
@@ -1616,8 +1636,9 @@ static void state_change(struct supplicant_task *task, DBusMessage *msg)
 		break;
 
 	case WPA_INACTIVE:
+		connman_network_set_connected(task->network, FALSE);
+
 		if (task->disconnecting == TRUE) {
-			connman_network_set_connected(task->network, FALSE);
 			connman_network_unref(task->network);
 			task->disconnecting = FALSE;
 
@@ -1699,7 +1720,7 @@ int supplicant_start(struct connman_device *device)
 	task->device = connman_device_ref(device);
 
 	task->created = FALSE;
-	task->noscan = FALSE;
+	task->scanning = FALSE;
 	task->state = WPA_INVALID;
 	task->disconnecting = FALSE;
 	task->pending_network = NULL;
@@ -1762,7 +1783,18 @@ int supplicant_scan(struct connman_device *device)
 		break;
 	}
 
+	task->scanning = TRUE;
+
 	err = initiate_scan(task);
+	if (err < 0) {
+		if (err == -EINPROGRESS)
+			return 0;
+
+		task->scanning = FALSE;
+		return err;
+	}
+
+	connman_device_set_scanning(task->device, TRUE);
 
 	return 0;
 }
@@ -1804,7 +1836,7 @@ int supplicant_disconnect(struct connman_network *network)
 		return -ENODEV;
 
 	if (task->disconnecting == TRUE)
-		return -EINPROGRESS;
+		return -EALREADY;
 
 	disable_network(task);
 
